/*
 * Copyright (c) 1991-2023 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and
 * its documentation is hereby granted, provided that this copyright
 * notice and license appear on all copies of the software.
 * 
 * STEP TOOLS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. STEP TOOLS
 * SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES.
 *
 * Author: Dave Loffredo (loffredo@steptools.com)
 */

#include <structural_frame_schema.h>
#include "geometry_utils.h"
#include "unit_utils.h"

// CIS/2 uses representation contexts unchanged from STEP.

// The geometric context information gives the number of dimensions
// (usually 3), length, angle, and solid angle units, and potentially
// a global uncertainty measure for comparing brep point locations.
//
// This information is held by a single representation context object
// that is an instance of the following subtypes:
//
//    - geometric_representation_context
//    - global_uncertainty_assigned_context
//    - global_unit_assigned_context (if an uncertainty is given)
//
representation_context * make_geometry_context (
    RoseDesign * design,
    const char * id,
    unsigned dims,
    named_unit * len_unit,
    named_unit * ang_unit,
    named_unit * solid_ang_unit,
    double length_uncertainty
    )
{
    geometric_representation_context * 	as_geo;
    global_unit_assigned_context * 		as_units;
    global_uncertainty_assigned_context * 	as_uncert;

    if (length_uncertainty == ROSE_NULL_REAL) {
	geometric_representation_context_and_global_unit_assigned_context * ctx = 
	    pnewIn (design) geometric_representation_context_and_global_unit_assigned_context;
	as_geo = ctx;
	as_units = ctx;
	as_uncert = 0;	// no uncertainty part
    }
    else {
	geometric_representation_context_and_global_uncertainty_assigned_context_and_global_unit_assigned_context * ctx = 
	    pnewIn (design) geometric_representation_context_and_global_uncertainty_assigned_context_and_global_unit_assigned_context;
	as_geo = ctx;
	as_units = ctx;
	as_uncert = ctx;
    }

    // Set the general geometric part of the context
    char buf[50]; 
    sprintf (buf, "%dD", dims);
    as_geo-> context_identifier ("ID1");
    as_geo-> context_type (buf);
    as_geo-> coordinate_space_dimension (dims);

    // Set the global units part of the context.  Each of the units is
    // wrapped in an instance of the unit select type.
    //
    unit * u;

    u = pnewIn(design) unit;
    u-> _named_unit (len_unit);
    as_units-> units()-> add (u);

    u = pnewIn(design) unit;
    u-> _named_unit (ang_unit);
    as_units-> units()-> add (u);

    u = pnewIn(design) unit;
    u-> _named_unit (solid_ang_unit);
    as_units-> units()-> add (u);

    // Set the uncertainty value if given
    if (as_uncert) {
	uncertainty_measure_with_unit * umwu = 
	    pnewIn(design) uncertainty_measure_with_unit;

	umwu-> value_component (pnewIn(design) measure_value);
	umwu-> value_component ()-> _length_measure (length_uncertainty);

	umwu-> unit_component (pnewIn(design) unit);
	umwu-> unit_component ()-> _named_unit (len_unit);

	umwu-> name ("DISTANCE_ACCURACY_VALUE");
	umwu-> description ("Maximum model space distance between geometric "
			    "entities at asserted connectivities");

	as_uncert-> uncertainty()-> add (umwu);
    }
    return as_geo;
}
